"""The Simple Management Protocol (SMP) Message base class."""

from __future__ import annotations

import itertools
import logging
from abc import ABC
from enum import IntEnum, unique
from typing import ClassVar, Type, TypeVar, cast

import cbor2
from pydantic import BaseModel, ConfigDict

from smp import header as smpheader
from smp.exceptions import SMPMalformed, SMPMismatchedGroupId

T = TypeVar("T", bound='_MessageBase')


_counter = itertools.count()
logger = logging.getLogger(__name__)


class _MessageBase(ABC, BaseModel):
    model_config = ConfigDict(extra="forbid", frozen=True)

    _OP: ClassVar[smpheader.OP]
    _FLAGS: ClassVar[smpheader.Flag] = smpheader.Flag(0)
    _GROUP_ID: ClassVar[smpheader.GroupId | smpheader.UserGroupId | smpheader.AnyGroupId]
    _COMMAND_ID: ClassVar[
        smpheader.AnyCommandId
        | smpheader.CommandId.ImageManagement
        | smpheader.CommandId.OSManagement
        | smpheader.CommandId.ShellManagement
        | smpheader.CommandId.Intercreate
        | smpheader.CommandId.FileManagement
    ]

    # This is is a dummy header that will be replaced in model_post_init
    header: smpheader.Header = None  # type: ignore
    version: smpheader.Version = smpheader.Version.V2
    sequence: int = None  # type: ignore
    smp_data: bytes = None  # type: ignore

    def __bytes__(self) -> bytes:
        return self.smp_data

    @property
    def BYTES(self) -> bytes:
        return self.smp_data

    @classmethod
    def loads(cls: Type[T], data: bytes) -> T:
        """Deserialize the SMP message."""
        message = cls(
            header=smpheader.Header.loads(data[: smpheader.Header.SIZE]),
            **cast(dict, cbor2.loads(data[smpheader.Header.SIZE :])),
            smp_data=data,
        )
        if message.header is None:  # pragma: no cover
            raise ValueError
        if message.header.group_id != cls._GROUP_ID:  # pragma: no cover
            raise SMPMismatchedGroupId(
                f"{cls.__name__} has {cls._GROUP_ID}, header has {message.header.group_id}"
            )
        return message

    @classmethod
    def load(cls: Type[T], header: smpheader.Header, data: dict) -> T:
        """Load an SMP header and CBOR dict."""
        if header.group_id != cls._GROUP_ID:  # pragma: no cover
            raise SMPMismatchedGroupId(
                f"{cls.__name__} has {cls._GROUP_ID}, header has {header.group_id}"
            )
        return cls(header=header, **data)

    def model_post_init(self, _: None) -> None:
        data_bytes = cbor2.dumps(
            self.model_dump(
                exclude_unset=True,
                exclude={'header', 'version', 'sequence', 'smp_data'},
                exclude_none=True,
            )
        )
        if self.header is None:  # create the header
            object.__setattr__(
                self,
                'header',
                smpheader.Header(
                    op=self._OP,
                    version=self.version,
                    flags=smpheader.Flag(self._FLAGS),
                    length=len(data_bytes),
                    group_id=self._GROUP_ID,
                    sequence=next(_counter) % 0xFF if self.sequence is None else self.sequence,
                    command_id=self._COMMAND_ID,
                ),
            )
            object.__setattr__(self, 'sequence', self.header.sequence)
        else:  # validate the header and update version & sequence
            if self.smp_data is None and self.header.length != len(data_bytes):
                raise SMPMalformed(
                    f"header.length {self.header.length} != len(data_bytes) {len(data_bytes)}"
                )
            if self.sequence is not None:  # pragma: no cover
                raise ValueError(
                    f"{self.sequence=} {self.header.sequence=} "
                    "Do not use the sequence attribute when the header is provided."
                )
            object.__setattr__(self, 'sequence', self.header.sequence)
            if self.version != self.header.version:
                logger.warning(
                    f"Overriding {self.version=} with {self.header.version=} "
                    "from the provided header."
                )
            object.__setattr__(self, 'version', self.header.version)
        if self.smp_data is None:
            object.__setattr__(self, 'smp_data', bytes(self.header) + data_bytes)


class Request(_MessageBase, ABC):
    ...


@unique
class ResponseType(IntEnum):
    """An SMP `Response` to an SMP `Request` must be `SUCCESS`, `ERROR_V1`, or `ERROR_V2`."""

    SUCCESS = 0
    ERROR_V1 = 1
    ERROR_V2 = 2


class Response(_MessageBase, ABC):
    RESPONSE_TYPE: ClassVar[ResponseType]


class ReadRequest(Request, ABC):
    """A read request from an SMP client to an SMP server."""

    _OP = smpheader.OP.READ


class ReadResponse(Response, ABC):
    """A response from an SMP server to an SMP client read request."""

    RESPONSE_TYPE = ResponseType.SUCCESS
    _OP = smpheader.OP.READ_RSP


class WriteRequest(Request, ABC):
    """A write request from an SMP client to an SMP server."""

    _OP = smpheader.OP.WRITE


class WriteResponse(Response, ABC):
    """A response from an SMP server to an SMP client write request."""

    RESPONSE_TYPE = ResponseType.SUCCESS
    _OP = smpheader.OP.WRITE_RSP



"""The Simple Management Protocol (SMP) Settings Management group."""

from __future__ import annotations

from enum import IntEnum, unique

import smp.error as smperr
import smp.header as smphdr
import smp.message as smpmsg


class ReadSettingRequest(smpmsg.ReadRequest):
    """Read setting."""

    _GROUP_ID = smphdr.GroupId.SETTINGS_MANAGEMENT
    _COMMAND_ID = smphdr.CommandId.SettingsManagement.READ_WRITE_SETTING

    name: str
    """The name of the setting to read."""

    max_size: int | None = None
    """The maximum size of the data to read."""


class ReadSettingResponse(smpmsg.ReadResponse):
    """Read setting success response."""

    _GROUP_ID = smphdr.GroupId.SETTINGS_MANAGEMENT
    _COMMAND_ID = smphdr.CommandId.SettingsManagement.READ_WRITE_SETTING

    val: bytes
    """Binary string of the returned data.

    Note that the underlying data type cannot be specified through this and must
    be known by the client.
    """

    max_size: int | None = None
    """The SMP server supports a smaller size than requested.

    Will be set if the maximum supported data size is smaller than the maximum
    requested data size, and contains the maximum data size which the device
    supports, equivalent to `CONFIG_MCUMGR_GRP_SETTINGS_NAME_LEN`.
    """


class WriteSettingRequest(smpmsg.WriteRequest):
    """Write setting."""

    _GROUP_ID = smphdr.GroupId.SETTINGS_MANAGEMENT
    _COMMAND_ID = smphdr.CommandId.SettingsManagement.READ_WRITE_SETTING

    name: str
    """The name of the setting to write."""

    val: bytes
    """Binary data to write."""


class WriteSettingResponse(smpmsg.WriteResponse):
    """Write setting success response."""

    _GROUP_ID = smphdr.GroupId.SETTINGS_MANAGEMENT
    _COMMAND_ID = smphdr.CommandId.SettingsManagement.READ_WRITE_SETTING


class DeleteSettingRequest(smpmsg.WriteRequest):
    """Delete setting."""

    _GROUP_ID = smphdr.GroupId.SETTINGS_MANAGEMENT
    _COMMAND_ID = smphdr.CommandId.SettingsManagement.DELETE_SETTING

    name: str
    """The name of the setting to delete."""


class DeleteSettingResponse(smpmsg.WriteResponse):
    """Delete setting success response."""

    _GROUP_ID = smphdr.GroupId.SETTINGS_MANAGEMENT
    _COMMAND_ID = smphdr.CommandId.SettingsManagement.DELETE_SETTING


class CommitSettingsRequest(smpmsg.WriteRequest):
    """Commit pending settings.

    Commit settings command allows committing all settings that have been set
    but not yet applied on a device.
    """

    _GROUP_ID = smphdr.GroupId.SETTINGS_MANAGEMENT
    _COMMAND_ID = smphdr.CommandId.SettingsManagement.COMMIT_SETTINGS


class CommitSettingsResponse(smpmsg.WriteResponse):
    """Commit pending settings success response."""

    _GROUP_ID = smphdr.GroupId.SETTINGS_MANAGEMENT
    _COMMAND_ID = smphdr.CommandId.SettingsManagement.COMMIT_SETTINGS


class LoadSettingsRequest(smpmsg.ReadRequest):
    """Load settings from persistent storage."""

    _GROUP_ID = smphdr.GroupId.SETTINGS_MANAGEMENT
    _COMMAND_ID = smphdr.CommandId.SettingsManagement.LOAD_SAVE_SETTINGS


class LoadSettingsResponse(smpmsg.ReadResponse):
    """Load settings from persistent storage success response."""

    _GROUP_ID = smphdr.GroupId.SETTINGS_MANAGEMENT
    _COMMAND_ID = smphdr.CommandId.SettingsManagement.LOAD_SAVE_SETTINGS


class SaveSettingsRequest(smpmsg.WriteRequest):
    """Save settings to persistent storage."""

    _GROUP_ID = smphdr.GroupId.SETTINGS_MANAGEMENT
    _COMMAND_ID = smphdr.CommandId.SettingsManagement.LOAD_SAVE_SETTINGS


class SaveSettingsResponse(smpmsg.WriteResponse):
    """Save settings to persistent storage success response."""

    _GROUP_ID = smphdr.GroupId.SETTINGS_MANAGEMENT
    _COMMAND_ID = smphdr.CommandId.SettingsManagement.LOAD_SAVE_SETTINGS


@unique
class SETTINGS_MGMT_ERR(IntEnum):
    OK = 0
    """No error, this is implied if there is no ret value in the response."""

    UNKNOWN = 1
    """Unknown error occurred."""

    KEY_TOO_LONG = 2
    """The provided key name is too long to be used."""

    KEY_NOT_FOUND = 3
    """The provided key name does not exist."""

    READ_NOT_SUPPORTED = 4
    """The provided key name does not support being read."""

    ROOT_KEY_NOT_FOUND = 5
    """The provided root key name does not exist."""

    WRITE_NOT_SUPPORTED = 6
    """The provided key name does not support being written."""

    DELETE_NOT_SUPPORTED = 7
    """The provided key name does not support being deleted."""


class SettingsManagementErrorV1(smperr.ErrorV1):
    _GROUP_ID = smphdr.GroupId.SETTINGS_MANAGEMENT


class SettingsManagementErrorV2(smperr.ErrorV2[SETTINGS_MGMT_ERR]):
    _GROUP_ID = smphdr.GroupId.SETTINGS_MANAGEMENT



"""The Simple Management Protocol (SMP) File Management group."""

from __future__ import annotations

from enum import IntEnum, auto, unique
from typing import Dict

from pydantic import BaseModel, ConfigDict

from smp import error, header, message


class FileDownloadRequest(message.ReadRequest):
    _GROUP_ID = header.GroupId.FILE_MANAGEMENT
    _COMMAND_ID = header.CommandId.FileManagement.FILE_DOWNLOAD_UPLOAD

    off: int
    name: str


class FileDownloadResponse(message.ReadResponse):
    _GROUP_ID = header.GroupId.FILE_MANAGEMENT
    _COMMAND_ID = header.CommandId.FileManagement.FILE_DOWNLOAD_UPLOAD

    off: int
    data: bytes
    len: int | None = None  # only mandatory when “off” is 0.


class FileUploadRequest(message.WriteRequest):
    _GROUP_ID = header.GroupId.FILE_MANAGEMENT
    _COMMAND_ID = header.CommandId.FileManagement.FILE_DOWNLOAD_UPLOAD

    off: int
    data: bytes
    name: str
    len: int | None = None  # this field is only mandatory when “off” is 0


class FileUploadResponse(message.WriteResponse):
    _GROUP_ID = header.GroupId.FILE_MANAGEMENT
    _COMMAND_ID = header.CommandId.FileManagement.FILE_DOWNLOAD_UPLOAD

    off: int


class FileStatusRequest(message.ReadRequest):
    _GROUP_ID = header.GroupId.FILE_MANAGEMENT
    _COMMAND_ID = header.CommandId.FileManagement.FILE_STATUS

    name: str


class FileStatusResponse(message.ReadResponse):
    _GROUP_ID = header.GroupId.FILE_MANAGEMENT
    _COMMAND_ID = header.CommandId.FileManagement.FILE_STATUS

    len: int


class FileHashChecksumRequest(message.ReadRequest):
    _GROUP_ID = header.GroupId.FILE_MANAGEMENT
    _COMMAND_ID = header.CommandId.FileManagement.FILE_HASH_CHECKSUM

    name: str
    type: str | None = None  # omit to use default.
    off: int | None = None  # 0 if not provided
    len: int | None = None  # full size if not provided


class FileHashChecksumResponse(message.ReadResponse):
    _GROUP_ID = header.GroupId.FILE_MANAGEMENT
    _COMMAND_ID = header.CommandId.FileManagement.FILE_HASH_CHECKSUM

    type: str
    off: int | None = None  # only present if not 0
    len: int
    output: int | bytes


class SupportedFileHashChecksumTypesRequest(message.ReadRequest):
    _GROUP_ID = header.GroupId.FILE_MANAGEMENT
    _COMMAND_ID = header.CommandId.FileManagement.SUPPORTED_FILE_HASH_CHECKSUM_TYPES


class HashChecksumType(BaseModel):
    model_config = ConfigDict(extra="forbid", frozen=True)

    format: int
    size: int


class SupportedFileHashChecksumTypesResponse(message.ReadResponse):
    _GROUP_ID = header.GroupId.FILE_MANAGEMENT
    _COMMAND_ID = header.CommandId.FileManagement.SUPPORTED_FILE_HASH_CHECKSUM_TYPES

    types: Dict[str, HashChecksumType]


class FileCloseRequest(message.WriteRequest):
    _GROUP_ID = header.GroupId.FILE_MANAGEMENT
    _COMMAND_ID = header.CommandId.FileManagement.FILE_CLOSE


class FileCloseResponse(message.WriteResponse):
    _GROUP_ID = header.GroupId.FILE_MANAGEMENT
    _COMMAND_ID = header.CommandId.FileManagement.FILE_CLOSE


@unique
class FS_MGMT_ERR(IntEnum):
    OK = 0
    """No error (success)."""

    UNKNOWN = auto()
    """ Unknown error. """

    FILE_INVALID_NAME = auto()
    """The specified file name is not valid."""

    FILE_NOT_FOUND = auto()
    """The specified file does not exist."""

    FILE_IS_DIRECTORY = auto()
    """The specified file is a directory, not a file. """

    FILE_OPEN_FAILED = auto()
    """Error occurred whilst attempting to open a file."""

    FILE_SEEK_FAILED = auto()
    """Error occurred whilst attempting to seek to an offset in a file. """

    FILE_READ_FAILED = auto()
    """Error occurred whilst attempting to read data from a file."""

    FILE_TRUNCATE_FAILED = auto()
    """Error occurred whilst trying to truncate file."""

    FILE_DELETE_FAILED = auto()
    """Error occurred whilst trying to delete file."""

    FILE_WRITE_FAILED = auto()
    """Error occurred whilst attempting to write data to a file."""

    FILE_OFFSET_NOT_VALID = auto()
    """The specified data offset is not valid, this could indicate that the file on the device has
    changed since the previous command. The length of the current file on the device is returned as
    "len", the user application needs to decide how to handle this (e.g. the hash of the file could
    be requested and compared with the hash of the length of the file being uploaded to see if they
    match or not)."""

    FILE_OFFSET_LARGER_THAN_FILE = auto()
    """The requested offset is larger than the size of the file on the device."""

    CHECKSUM_HASH_NOT_FOUND = auto()
    """The requested checksum or hash type was not found or is not supported by this build."""

    MOUNT_POINT_NOT_FOUND = auto()
    """The specified mount point was not found or is not mounted."""

    READ_ONLY_FILESYSTEM = auto()
    """The specified mount point is that of a read-only filesystem."""

    FILE_EMPTY = auto()
    """The operation cannot be performed because the file is empty with no contents. """


class FileSystemManagementErrorV1(error.ErrorV1):
    _GROUP_ID = header.GroupId.FILE_MANAGEMENT


class FileSystemManagementErrorV2(error.ErrorV2[FS_MGMT_ERR]):
    _GROUP_ID = header.GroupId.FILE_MANAGEMENT



"""The Simple Management Protocol (SMP) Image Management group."""

from __future__ import annotations

from enum import IntEnum, auto, unique
from typing import Generator, List

from pydantic import BaseModel, ConfigDict, ValidationInfo, field_validator

from smp import error, header, message


class HashBytes(bytes):  # pragma: no cover
    """Only to print something useful to the console."""

    def __rich_repr__(self) -> Generator[str, None, None]:
        yield self.hex().upper()


class ImageState(BaseModel):
    model_config = ConfigDict(extra="forbid", frozen=True, arbitrary_types_allowed=True)

    slot: int
    version: str
    image: int | None = None
    hash: HashBytes | bytes | None = None
    bootable: bool | None = None
    pending: bool | None = None
    confirmed: bool | None = None
    active: bool | None = None
    permanent: bool | None = None

    @field_validator("hash")
    @classmethod
    def cast_bytes(cls, v: bytes | None, _: ValidationInfo) -> HashBytes | None:
        if v is None:
            return None
        return HashBytes(v)


class ImageStatesReadRequest(message.ReadRequest):
    _GROUP_ID = header.GroupId.IMAGE_MANAGEMENT
    _COMMAND_ID = header.CommandId.ImageManagement.STATE


class ImageStatesReadResponse(message.ReadResponse):
    _GROUP_ID = header.GroupId.IMAGE_MANAGEMENT
    _COMMAND_ID = header.CommandId.ImageManagement.STATE

    images: List[ImageState]
    splitStatus: int | None = None


class ImageStatesWriteRequest(message.WriteRequest):
    _GROUP_ID = header.GroupId.IMAGE_MANAGEMENT
    _COMMAND_ID = header.CommandId.ImageManagement.STATE

    hash: bytes
    confirm: bool = False


class ImageStatesWriteResponse(ImageStatesReadResponse):
    pass


class ImageUploadWriteRequest(message.WriteRequest):
    _GROUP_ID = header.GroupId.IMAGE_MANAGEMENT
    _COMMAND_ID = header.CommandId.ImageManagement.UPLOAD

    off: int
    data: bytes
    image: int | None = None  # 'required' when off == 0
    len: int | None = None  # required when off == 0
    sha: bytes | None = None  # should be sent when off == 0
    upgrade: bool | None = None  # allowed when off == 0


class ImageUploadWriteResponse(message.WriteResponse):
    _GROUP_ID = header.GroupId.IMAGE_MANAGEMENT
    _COMMAND_ID = header.CommandId.ImageManagement.UPLOAD

    off: int | None = None
    """The portion of the upload that has been completed, in 8-bit bytes.

    This is the offset of the next byte to be written. If the offset is equal to
    the length of the image, the upload is complete.
    """

    match: bool | None = None
    """Indicates if the uploaded data successfully matches the provided SHA256.

    Only sent in the final packet if CONFIG_IMG_ENABLE_IMAGE_CHECK is enabled.
    """

    rc: int | None = None
    """Legacy field that contains a return code; possibly `MGMT_ERR`.

    This field may be present on old SMP server implementations or new SMP
    server implementations that have set
    `CONFIG_MCUMGR_SMP_LEGACY_RC_BEHAVIOUR=y` for backwards compatibility with
    old SMP clients.

    Note that we are not validating this field because we don't necessarily
    trust the server to send us valid values. If this value is present, then it
    indicates use of an SMP server that is out of spec and interpretation of the
    value should be done with reference to that server's source code, rather
    that the SMP specification.

    Zephyr source code reference: https://github.com/zephyrproject-rtos/zephyr/blob/91a1e706535b2f99433280513c5bc66dfb918506/subsys/mgmt/mcumgr/grp/img_mgmt/src/img_mgmt.c#L397-L400
    """  # noqa: E501


class ImageEraseRequest(message.WriteRequest):
    _GROUP_ID = header.GroupId.IMAGE_MANAGEMENT
    _COMMAND_ID = header.CommandId.ImageManagement.ERASE

    slot: int | None = None
    """The slot to erase. If not provided, slot 1 will be erased."""


class ImageEraseResponse(message.WriteResponse):
    _GROUP_ID = header.GroupId.IMAGE_MANAGEMENT
    _COMMAND_ID = header.CommandId.ImageManagement.ERASE


@unique
class IMG_MGMT_ERR(IntEnum):
    OK = 0
    """No error, this is implied if there is no ret value in the response"""

    UNKNOWN = auto()
    """Unknown error occurred."""

    FLASH_CONFIG_QUERY_FAIL = auto()
    """Failed to query flash area configuration."""

    NO_IMAGE = auto()
    """There is no image in the slot."""

    NO_TLVS = auto()
    """The image in the slot has no TLVs (tag, length, value)."""

    INVALID_TLV = auto()
    """The image in the slot has an invalid TLV type and/or length."""

    TLV_MULTIPLE_HASHES_FOUND = auto()
    """The image in the slot has multiple hash TLVs, which is invalid."""

    TLV_INVALID_SIZE = auto()
    """The image in the slot has an invalid TLV size."""

    HASH_NOT_FOUND = auto()
    """The image in the slot does not have a hash TLV, which is required."""

    NO_FREE_SLOT = auto()
    """There is no free slot to place the image."""

    FLASH_OPEN_FAILED = auto()
    """Flash area opening failed."""

    FLASH_READ_FAILED = auto()
    """Flash area reading failed."""

    FLASH_WRITE_FAILED = auto()
    """Flash area writing failed."""

    FLASH_ERASE_FAILED = auto()
    """Flash area erase failed."""

    INVALID_SLOT = auto()
    """The provided slot is not valid."""

    NO_FREE_MEMORY = auto()
    """Insufficient heap memory (malloc failed)."""

    FLASH_CONTEXT_ALREADY_SET = auto()
    """The flash context is already set."""

    FLASH_CONTEXT_NOT_SET = auto()
    """The flash context is not set."""

    FLASH_AREA_DEVICE_NULL = auto()
    """The device for the flash area is NULL."""

    INVALID_PAGE_OFFSET = auto()
    """The offset for a page number is invalid."""

    INVALID_OFFSET = auto()
    """The offset parameter was not provided and is required."""

    INVALID_LENGTH = auto()
    """The length parameter was not provided and is required."""

    INVALID_IMAGE_HEADER = auto()
    """The image length is smaller than the size of an image header."""

    INVALID_IMAGE_HEADER_MAGIC = auto()
    """The image header magic value does not match the expected value."""

    INVALID_HASH = auto()
    """The hash parameter provided is not valid."""

    INVALID_FLASH_ADDRESS = auto()
    """The image load address does not match the address of the flash area."""

    VERSION_GET_FAILED = auto()
    """Failed to get version of currently running application."""

    CURRENT_VERSION_IS_NEWER = auto()
    """The currently running application is newer than the version being uploaded."""

    IMAGE_ALREADY_PENDING = auto()
    """There is already an image operating pending."""

    INVALID_IMAGE_VECTOR_TABLE = auto()
    """The image vector table is invalid."""

    INVALID_IMAGE_TOO_LARGE = auto()
    """The image it too large to fit."""

    INVALID_IMAGE_DATA_OVERRUN = auto()
    """The amount of data sent is larger than the provided image size."""

    IMAGE_CONFIRMATION_DENIED = auto()
    """Confirmation of image has been denied"""

    IMAGE_SETTING_TEST_TO_ACTIVE_DENIED = auto()
    """Setting test to active slot is not allowed"""


class ImageManagementErrorV1(error.ErrorV1):
    _GROUP_ID = header.GroupId.IMAGE_MANAGEMENT


class ImageManagementErrorV2(error.ErrorV2[IMG_MGMT_ERR]):
    _GROUP_ID = header.GroupId.IMAGE_MANAGEMENT



"""The Simple Management Protocol (SMP) Statistics Management group."""


from enum import IntEnum, unique
from typing import Dict, Tuple

import smp.error as smperr
import smp.header as smphdr
import smp.message as smpmsg


class GroupDataRequest(smpmsg.ReadRequest):
    _GROUP_ID = smphdr.GroupId.STATISTICS_MANAGEMENT
    _COMMAND_ID = smphdr.CommandId.StatisticsManagement.GROUP_DATA

    name: str


class GroupDataResponse(smpmsg.ReadResponse):
    _GROUP_ID = smphdr.GroupId.STATISTICS_MANAGEMENT
    _COMMAND_ID = smphdr.CommandId.StatisticsManagement.GROUP_DATA

    name: str
    fields: Dict[str, int]


class ListOfGroupsRequest(smpmsg.ReadRequest):
    _GROUP_ID = smphdr.GroupId.STATISTICS_MANAGEMENT
    _COMMAND_ID = smphdr.CommandId.StatisticsManagement.LIST_OF_GROUPS


class ListOfGroupsResponse(smpmsg.ReadResponse):
    _GROUP_ID = smphdr.GroupId.STATISTICS_MANAGEMENT
    _COMMAND_ID = smphdr.CommandId.StatisticsManagement.LIST_OF_GROUPS

    stat_list: Tuple[str, ...]


@unique
class STAT_MGMT_ERR(IntEnum):
    OK = 0
    """No error, this is implied if there is no ret value in the response."""

    UNKNOWN = 1
    """Unknown error occurred."""

    ERR_INVALID_GROUP = 2
    """The provided statistic group name was not found."""

    ERR_INVALID_STAT_NAME = 3
    """The provided statistic name was not found."""

    ERR_INVALID_STAT_SIZE = 4
    """The size of the statistic cannot be handled."""

    ERR_WALK_ABORTED = 5
    """Walk through of statistics was aborted."""


class StatisticsManagementErrorV1(smperr.ErrorV1):
    _GROUP_ID = smphdr.GroupId.STATISTICS_MANAGEMENT


class StatisticsManagementErrorV2(smperr.ErrorV2[STAT_MGMT_ERR]):
    _GROUP_ID = smphdr.GroupId.STATISTICS_MANAGEMENT
